import 'package:equatable/equatable.dart';

abstract class CreateUnitState extends Equatable {
  const CreateUnitState();

  @override
  List<Object> get props => [];
}

class InitialState extends CreateUnitState {}

class LoadingState extends CreateUnitState {}

class CreateRoomState extends CreateUnitState {
  final List<String> roomList;

  const CreateRoomState({required this.roomList});

  @override
  List<Object> get props => [roomList];
}

class RoomsSavedSuccessfully extends CreateUnitState {}

class ErrorState extends CreateUnitState {
  final String message;

  const ErrorState({required this.message});

  @override
  List<Object> get props => [message];
}
